@@ -56,6 +56,7 @@ static void rb_android_device_source_init (RBDeviceSourceInterface *interface);
 static void rb_android_transfer_target_init (RBTransferTargetInterface *interface);
 
 static void find_music_dirs (RBAndroidSource *source);
+static void rescan_music_dirs (RBAndroidSource *source);
 static void update_free_space_next (RBAndroidSource *source);
 
 enum
@@ -94,6 +95,10 @@ typedef struct
 	GList *query_storage;
 	guint64 storage_free_space_next;
 	guint64 storage_capacity_next;
+	guint rescan_id;
+
+	GtkWidget *grid;
+	GtkWidget *info_bar;
 } RBAndroidSourcePrivate;
 
 G_DEFINE_DYNAMIC_TYPE_EXTENDED (
@@ -121,7 +126,8 @@ free_space_cb (GObject *obj, GAsyncResult *res, gpointer data)
 	} else {
 		priv->storage_free_space_next += g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
 		priv->storage_capacity_next += g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
-		rb_debug ("capacity: %lu, free space: %lu", priv->storage_capacity_next, priv->storage_free_space_next);
+		rb_debug ("capacity: %" G_GUINT64_FORMAT ", free space: %" G_GUINT64_FORMAT,
+		          priv->storage_capacity_next, priv->storage_free_space_next);
 	}
 
 	priv->query_storage = priv->query_storage->next;
@@ -154,6 +160,11 @@ update_free_space (RBAndroidSource *source)
 		return;
 	}
 
+	if (priv->storage == NULL) {
+		rb_debug ("no storage to query");
+		return;
+	}
+
 	priv->storage_free_space_next = 0;
 	priv->storage_capacity_next = 0;
 	priv->query_storage = priv->storage;
@@ -165,10 +176,34 @@ static void
 music_dirs_done (RBAndroidSource *source)
 {
 	RBAndroidSourcePrivate *priv = GET_PRIVATE(source);
-	rb_debug ("finished checking for music dirs");
-	rhythmdb_import_job_start (priv->import_job);
 
-	update_free_space (source);
+	if (priv->scanned > 1) {
+		gtk_widget_hide (priv->info_bar);
+		rhythmdb_import_job_start (priv->import_job);
+
+		if (priv->rescan_id != 0) {
+			g_source_remove (priv->rescan_id);
+		}
+
+		if (priv->storage != NULL) {
+			rb_debug ("finished checking for music dirs");
+			update_free_space (source);
+		} else {
+			rb_debug ("no music dirs found (%d)", priv->scanned);
+		}
+	} else {
+		GtkWidget *label;
+
+		rb_debug ("no storage areas found");
+		if (gtk_widget_get_visible (priv->info_bar) == FALSE) {
+			label = gtk_label_new (_("No storage areas found on this device. You may need to unlock it and enable MTP."));
+			gtk_container_add (GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar))), label);
+			gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->info_bar), GTK_MESSAGE_INFO);
+			gtk_widget_show_all (priv->info_bar);
+		}
+		if (priv->rescan_id == 0)
+			priv->rescan_id = g_timeout_add_seconds (5, (GSourceFunc) rescan_music_dirs, source);
+	}
 }
 
 
@@ -280,6 +315,23 @@ find_music_dirs (RBAndroidSource *source)
 	g_object_unref (dir);
 }
 
+static void
+rescan_music_dirs (RBAndroidSource *source)
+{
+	RBAndroidSourcePrivate *priv = GET_PRIVATE (source);
+	GMount *mount;
+	GFile *root;
+
+	g_object_get (source, "mount", &mount, NULL);
+	root = g_mount_get_root (mount);
+	g_object_unref (mount);
+
+	priv->scanned = 0;
+	g_queue_push_tail (&priv->to_scan, root);
+
+	find_music_dirs (source);
+}
+
 static void
 import_complete_cb (RhythmDBImportJob *job, int total, RBAndroidSource *source)
 {
@@ -725,6 +777,12 @@ impl_delete_thyself (RBDisplayPage *page)
 	RB_DISPLAY_PAGE_CLASS (rb_android_source_parent_class)->delete_thyself (page);
 }
 
+static void
+impl_pack_content (RBBrowserSource *source, GtkWidget *content)
+{
+	RBAndroidSourcePrivate *priv = GET_PRIVATE (source);
+	gtk_grid_attach (GTK_GRID (priv->grid), content, 0, 1, 1, 1);
+}
 
 static void
 rb_android_source_init (RBAndroidSource *source)
@@ -741,10 +799,18 @@ impl_constructed (GObject *object)
 	RBShell *shell;
 	char **output_formats;
 
-	RB_CHAIN_GOBJECT_METHOD (rb_android_source_parent_class, constructed, object);
 	source = RB_ANDROID_SOURCE (object);
-
 	priv = GET_PRIVATE (source);
+	priv->grid = gtk_grid_new ();
+
+	RB_CHAIN_GOBJECT_METHOD (rb_android_source_parent_class, constructed, object);
+
+	priv->info_bar = gtk_info_bar_new ();
+	gtk_grid_attach (GTK_GRID (priv->grid), priv->info_bar, 0, 0, 1, 1);
+
+	gtk_container_add (GTK_CONTAINER (source), priv->grid);
+	gtk_widget_show_all (priv->grid);
+	gtk_widget_hide (priv->info_bar);
 
 	rb_device_source_set_display_details (RB_DEVICE_SOURCE (source));
 
@@ -900,6 +966,7 @@ rb_android_source_class_init (RBAndroidSourceClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_class = RB_BROWSER_SOURCE_CLASS (klass);
 	RBMediaPlayerSourceClass *mps_class = RB_MEDIA_PLAYER_SOURCE_CLASS (klass);
 
 	object_class->set_property = impl_set_property;
@@ -911,6 +978,8 @@ rb_android_source_class_init (RBAndroidSourceClass *klass)
 	page_class->delete_thyself = impl_delete_thyself;
 	page_class->selected = impl_selected;
 
+	browser_class->pack_content = impl_pack_content;
+
 	source_class->can_delete = impl_can_delete;
 	source_class->delete_selected = impl_delete_selected;
 	source_class->can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
